const amqp = require('amqplib');
const cfg = require('../config/cfg.json');
const fsExtra = require('fs-extra');
const fs = require('fs');
const path = require('path');
const glbCompressor = require('../util/glbCompressor.js');
const glbSimplifier = require('../util/glbSimplifier.js');
const producer = require('./producer.js');

async function run() {
    await producer.init(); // !IMPORTANT!
    await glbSimplifier.init() //!IMPORTANT!
    let connection = await amqp.connect(cfg.rabbitmq.url);
    const channel = await connection.createChannel();
    channel.prefetch(cfg.rabbitmq.consumer.prefetch);
    await channel.assertQueue(cfg.rabbitmq.consumer.queue);
    console.log('-------------------------------\n',
        'Consumer started with prefetch: ' + cfg.rabbitmq.consumer.prefetch,
        '\n-------------------------------\n')
    channel.consume(
        cfg.rabbitmq.queue, async (msg) => {
            if (msg !== null) {
                let json = JSON.parse(msg.content.toString());
                let filename = json.file;
                let targetName = filename.replace(".glb", ".drc");
                console.log(filename)
                try {
                    if (fs.statSync(path.join(__dirname, "../queue/", filename)).size / (1024 * 1024) > cfg.maxMB) {
                        console.log("Simplifying model...")
                        await glbSimplifier.run(filename);
                    }
                    console.log('Compressing to drc...')
                    await glbCompressor.convert(filename, targetName);
                    await uploadToEndpoint(json.endpoint, targetName);
                    producer.sendResponseToViewer(json.author, 'system', 'Ваш файл был удачно обработан!');
                    await fsExtra.remove(path.join(__dirname, "../ready/", targetName));
                    await fsExtra.remove(path.join(__dirname, "../queue/", filename));
                    console.log('Done!')
                    channel.ack(msg);
                } catch (e) {
                    console.log('\n!-- ERROR OCCURRED --!\n\n',
                        e + '\n');
                    try {
                        await fsExtra.remove(path.join(__dirname, "../ready/", targetName));
                        console.log('Ready file ' + targetName + ' cleared!');
                    } catch (e) {/* ignore */
                    }
                    if (msg.fields.redelivered) {
                        try {
                            await fsExtra.remove(path.join(__dirname, "../queue/", filename));
                            console.log('File ' + filename + ' cleared!');
                        } catch (e) {/* ignore */}
                        console.log('\n!-- MESSAGE FAILED TWICE AND WILL BE DESTROYED --!\n');
                        producer.sendResponseToViewer(json.author, 'system', 'Ошибка при обпаботке вашего файла!')
                        channel.nack(msg, false, false);
                    } else {
                        console.log('\n!-- MESSAGE WILL BE RETURNED TO THE QUEUE ONE MORE TIME --!\n')
                        channel.nack(msg, false, true);
                    }
                }
            } else {
                console.log('Consumer cancelled by server');
            }
        });
}

async function uploadToEndpoint(endpoint, filename) {
    console.log("Uploading...")
    let data = Buffer.from(fsExtra.readFileSync(path.join(__dirname, "../ready/", filename)));
    await fetch(endpoint, {
        method: 'PUT',
        headers: {'Content-Type': 'application/octet'},
        body: data
    })
}

module.exports = {
    run
}