const gltfPipeline = require('gltf-pipeline');
const fsExtra = require('fs-extra');
const path = require('path');
const cfg = require('../config/cfg.json');

const options = {
    dracoOptions: {
        quantizePositionBits: cfg.draco.quantizeGenericBits,
        quantizeNormalBits: cfg.draco.quantizeNormalBits,
        quantizeTexcoordBits: cfg.draco.quantizeTexcoordBits,
        quantizeGenericBits: cfg.draco.quantizeGenericBits,
        compressionLevel: cfg.draco.compressionLevel
    }
}

async function convert(filename, targetName) {
    let file = fsExtra.readFileSync(path.resolve(path.join(__dirname, "../queue/", filename)));
    return new Promise((resolve) => {
        gltfPipeline.processGlb(file, options).then(result => {
            fsExtra.writeFileSync(path.resolve(path.join(__dirname, "../ready/", targetName)), result.glb);
            resolve()
        }).catch(e => {
            console.log("ERROR OCCURRED WHILE COMPRESSING!\nTrying to use low compression...");
            gltfPipeline.processGlb(file, {}).then(result => {
                fsExtra.writeFileSync(path.resolve(path.join(__dirname, "../ready/", targetName)), result.glb);
                resolve()
            }).catch(err => {
                console.log("ERROR OCCURRED WHILE LOW COMPRESSING!")
                resolve();
            })
        }) ;
    });
}


module.exports = {convert}